window.Gleam.initSearch([{"doc":"glemplate","title":"glemplate","content":"# glemplate\n\n[![Package Version](https://img.shields.io/hexpm/v/glemplate)](https://hex.pm/packages/glemplate)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glemplate/)\n\nGlemplate is a simplistic template engine for Gleam. The aim was to learn how to make such a thing\nand fill simple use cases. At this stage, it can be used for simple plain text and HTML templates,\nand for other contexts by supplying a custom encoding function.\n\n## Quick start\n\n```gleam\nimport gleam/dict\nimport glemplate/parser\nimport glemplate/html\nimport glemplate/assigns\n\nlet template = &quot;&lt;b&gt;Welcome, &lt;%= name %&gt;!&lt;/b&gt;&quot;\nlet p = parser.new()\nlet assert Ok(tpl) = parser.parse_to_template(template, &quot;input.html.glemp&quot;, p)\nlet assigns = assigns.from_list([#(&quot;name&quot;, assigns.String(&quot;&lt;Nicd&gt;&quot;))])\nlet template_cache = dict.new()\nhtml.render(tpl, assigns, template_cache) // &quot;&lt;b&gt;Welcome, &amp;lt;Nicd&amp;gt;!&lt;/b&gt;&quot;\n```\n\nThe main modules:\n\n- [glemplate/parser](glemplate/parser.html): The parser implements and documents the string\n  template syntax.\n- [glemplate/ast](glemplate/ast.html): The abstract syntax tree (AST) is the data format of the\n  parsed templates.\n- [glemplate/renderer](glemplate/renderer.html): The renderer takes in the parsed AST, input data,\n  and a cache of templates, and renders the template into an output string builder.\n- [glemplate/html](glemplate/html.html) and [glemplate/text](glemplate/text.html) have helper\n  functions for rendering HTML and plain text templates.\n\nUsing templates with Glemplate has two main things to deal with:\n\n1. parsing templates into AST with the parser, and\n2. rendering that AST into some output with assigns (input data).\n\nSince Gleam has no metaprogramming, the parsing of templates needs to be done at startup time (or\nwhenever deemed necessary when templates have changed). The resulting AST should be stored for\nlater use in e.g. ETS, process state, persistent_term…\n\nGlemplate does not offer any tooling for reading templates currently. You will need to use\nwhatever method appropriate for your situation: reading from files, compiling the strings in the\napp, reading from a database…\n\nWhen stored in files, it&#39;s suggested that templates use the file naming style\n`name.&lt;type&gt;.glemp`, where `&lt;type&gt;` signifies the type of content rendered from the template. E.g.\n`user.html.glemp`.\n\n## Limitations\n\n- Static values in templates in place of variables aren&#39;t supported.\n- It&#39;s possible to create infinite loops with child templates, it&#39;s not recommended to give access\n  to writing templates to untrusted users.\n- Calling functions from templates is not supported.\n\n## Installation\n\nThis package can be added to your Gleam project:\n\n```sh\ngleam add glemplate\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/glemplate&gt;.\n","url":"index.html"},{"doc":"glemplate/ast","title":"DynamicNode","content":"pub type DynamicNode {\n  Output(Var)\n  RawOutput(Var)\n  If(Var, if_true: NodeList, if_false: NodeList)\n  Iter(over: Var, binding: VarName, NodeList)\n  Render(tpl: TemplateName, assigns_map: List(#(Var, VarName)))\n}\n A dynamic node is used for dynamic behaviour in a template.\nOutput(Var)\n Output the variable, encoded.\n\nRawOutput(Var)\n Output the variable, without encoding.\n\nIf(Var, if_true: NodeList, if_false: NodeList)\n Render a node list based on if the assign is truthy.\n\n Truthy values are all values except `False`, `0`, or empty strings, lists,\n or dicts.\n\nIter(over: Var, binding: VarName, NodeList)\n Iterate over the variable, bind it to a new name, and render the node list\n for every item.\n\nRender(tpl: TemplateName, assigns_map: List(#(Var, VarName)))\n Render the given template, using the assigns mapping. The assigns mapping\n is a list of tuples `(from, to)`, where the `from` named assigns are\n available with `to` names in the child template.\n\nSynonyms:\nglemplate/ast.DynamicNode\nglemplate/ast DynamicNode","url":"glemplate/ast.html#DynamicNode"},{"doc":"glemplate/ast","title":"Node","content":"pub type Node {\n  Text(String)\n  Dynamic(DynamicNode)\n  Nodes(nodes: NodeList)\n}\n\nText(String)\n\n\nDynamic(DynamicNode)\n\n\nNodes(nodes: NodeList)\n\n\nSynonyms:\nglemplate/ast.Node\nglemplate/ast Node","url":"glemplate/ast.html#Node"},{"doc":"glemplate/ast","title":"NodeList","content":"pub type NodeList =\n  List(Node)\n\n\nSynonyms:\nglemplate/ast.NodeList\nglemplate/ast NodeList","url":"glemplate/ast.html#NodeList"},{"doc":"glemplate/ast","title":"Template","content":"pub type Template {\n  Template(name: TemplateName, nodes: NodeList)\n}\n\nTemplate(name: TemplateName, nodes: NodeList)\n\n\nSynonyms:\nglemplate/ast.Template\nglemplate/ast Template","url":"glemplate/ast.html#Template"},{"doc":"glemplate/ast","title":"TemplateName","content":"pub type TemplateName =\n  String\n A reference to a template with the given name.\n\nSynonyms:\nglemplate/ast.TemplateName\nglemplate/ast TemplateName","url":"glemplate/ast.html#TemplateName"},{"doc":"glemplate/ast","title":"Var","content":"pub type Var {\n  Assign(name: VarName)\n  FieldAccess(container: Var, field: String)\n}\n A reference to a variable input.\nAssign(name: VarName)\n Reference to an assign with the given name.\n\nFieldAccess(container: Var, field: String)\n Accessing a field of an associative container.\n\nSynonyms:\nglemplate/ast.Var\nglemplate/ast Var","url":"glemplate/ast.html#Var"},{"doc":"glemplate/ast","title":"VarName","content":"pub type VarName =\n  String\n\n\nSynonyms:\nglemplate/ast.VarName\nglemplate/ast VarName","url":"glemplate/ast.html#VarName"},{"doc":"glemplate/ast","title":"glemplate/ast","content":" The abstract syntax tree that templates are stored as in memory. This AST\n can be rendered with the renderer module.","url":"glemplate/ast.html"},{"doc":"glemplate/assigns","title":"AssignData","content":"pub type AssignData {\n  String(String)\n  Int(Int)\n  Lazy(LazyFn)\n  Bool(Bool)\n  Dict(Dict(VarName, AssignData))\n  List(List(AssignData))\n}\n Data in an assign. Note that only `String` and `Int` are stringifiable\n values.\nString(String)\n\n\nInt(Int)\n\n\nLazy(LazyFn)\n A function to execute that returns assign data on demand.\n\nBool(Bool)\n\n\nDict(Dict(VarName, AssignData))\n\n\nList(List(AssignData))\n\n\nSynonyms:\nglemplate/assigns.AssignData\nglemplate/assigns AssignData","url":"glemplate/assigns.html#AssignData"},{"doc":"glemplate/assigns","title":"Assigns","content":"pub type Assigns =\n  Dict(VarName, AssignData)\n Assigns given to a template for rendering dynamic content.\n\nSynonyms:\nglemplate/assigns.Assigns\nglemplate/assigns Assigns","url":"glemplate/assigns.html#Assigns"},{"doc":"glemplate/assigns","title":"LazyFn","content":"pub type LazyFn =\n  fn() -&gt; AssignData\n Lazy function that should return assign data when executed.\n\nSynonyms:\nglemplate/assigns.LazyFn\nglemplate/assigns LazyFn","url":"glemplate/assigns.html#LazyFn"},{"doc":"glemplate/assigns","title":"add_bool","content":"pub fn add_bool(\n  assigns: Dict(String, AssignData),\n  name: String,\n  value: Bool,\n) -&gt; Dict(String, AssignData)\n\nSynonyms:\nglemplate/assigns.add_bool\nglemplate/assigns add_bool","url":"glemplate/assigns.html#add_bool"},{"doc":"glemplate/assigns","title":"add_dict","content":"pub fn add_dict(\n  assigns: Dict(String, AssignData),\n  name: String,\n  value: Dict(String, AssignData),\n) -&gt; Dict(String, AssignData)\n\nSynonyms:\nglemplate/assigns.add_dict\nglemplate/assigns add_dict","url":"glemplate/assigns.html#add_dict"},{"doc":"glemplate/assigns","title":"add_int","content":"pub fn add_int(\n  assigns: Dict(String, AssignData),\n  name: String,\n  value: Int,\n) -&gt; Dict(String, AssignData)\n\nSynonyms:\nglemplate/assigns.add_int\nglemplate/assigns add_int","url":"glemplate/assigns.html#add_int"},{"doc":"glemplate/assigns","title":"add_lazy","content":"pub fn add_lazy(\n  assigns: Dict(String, AssignData),\n  name: String,\n  value: fn() -&gt; AssignData,\n) -&gt; Dict(String, AssignData)\n\nSynonyms:\nglemplate/assigns.add_lazy\nglemplate/assigns add_lazy","url":"glemplate/assigns.html#add_lazy"},{"doc":"glemplate/assigns","title":"add_list","content":"pub fn add_list(\n  assigns: Dict(String, AssignData),\n  name: String,\n  value: List(AssignData),\n) -&gt; Dict(String, AssignData)\n\nSynonyms:\nglemplate/assigns.add_list\nglemplate/assigns add_list","url":"glemplate/assigns.html#add_list"},{"doc":"glemplate/assigns","title":"add_string","content":"pub fn add_string(\n  assigns: Dict(String, AssignData),\n  name: String,\n  value: String,\n) -&gt; Dict(String, AssignData)\n\nSynonyms:\nglemplate/assigns.add_string\nglemplate/assigns add_string","url":"glemplate/assigns.html#add_string"},{"doc":"glemplate/assigns","title":"from_list","content":"pub fn from_list(\n  list: List(#(String, AssignData)),\n) -&gt; Dict(String, AssignData)\n\nSynonyms:\nglemplate/assigns.from_list\nglemplate/assigns from_list","url":"glemplate/assigns.html#from_list"},{"doc":"glemplate/assigns","title":"new","content":"pub fn new() -&gt; Dict(String, AssignData)\n\nSynonyms:\nglemplate/assigns.new\nglemplate/assigns new","url":"glemplate/assigns.html#new"},{"doc":"glemplate/assigns","title":"glemplate/assigns","content":" Assigns are used to insert dynamic data into a template. This module\n contains convenience functions for creating assigns, but you may also\n create the dict manually with Gleam&#39;s dict functions if you so wish.","url":"glemplate/assigns.html"},{"doc":"glemplate/renderer","title":"EncoderFn","content":"pub type EncoderFn =\n  fn(String) -&gt; StringTree\n Function to encode potentially dangerous contents into the template in a\n safe way. E.g. encode text with HTML entities in an HTML template.\n\nSynonyms:\nglemplate/renderer.EncoderFn\nglemplate/renderer EncoderFn","url":"glemplate/renderer.html#EncoderFn"},{"doc":"glemplate/renderer","title":"RenderError","content":"pub type RenderError {\n  AssignNotFound(assign: ast.Var, assigns: Assigns)\n  AssignNotIterable(assign: ast.Var, assigns: Assigns)\n  AssignNotStringifiable(assign: ast.Var, assigns: Assigns)\n  AssignFieldNotFound(\n    assign: ast.Var,\n    field: String,\n    assigns: Assigns,\n  )\n  AssignNotFieldAccessible(assign: ast.Var, assigns: Assigns)\n  ChildTemplateNotFound(tpl_name: ast.TemplateName)\n}\n\nAssignNotFound(assign: ast.Var, assigns: Assigns)\n A referenced assign was not found.\n\nAssignNotIterable(assign: ast.Var, assigns: Assigns)\n A referenced assign was not iterable.\n\nAssignNotStringifiable(assign: ast.Var, assigns: Assigns)\n A referenced assign could not be stringified.\n\nAssignFieldNotFound(\n  assign: ast.Var,\n  field: String,\n  assigns: Assigns,\n)\n A referenced assign didn&#39;t have the requested field.\n\nAssignNotFieldAccessible(assign: ast.Var, assigns: Assigns)\n A referenced assign couldn&#39;t be accessed with field access.\n\nChildTemplateNotFound(tpl_name: ast.TemplateName)\n A referenced child template was not found in the template cache.\n\nSynonyms:\nglemplate/renderer.RenderError\nglemplate/renderer RenderError","url":"glemplate/renderer.html#RenderError"},{"doc":"glemplate/renderer","title":"RenderOptions","content":"pub type RenderOptions {\n  RenderOptions(\n    encoder: EncoderFn,\n    template_cache: TemplateCache,\n  )\n}\n Rendering options:\n\n - `encoder`: Encoder function to use, it should encode dynamic content in a\n   safe way for this template type.\n - `template_cache`: A cache of templates that child templates are looked up\n   from, when using Render nodes.\nRenderOptions(encoder: EncoderFn, template_cache: TemplateCache)\n\n\nSynonyms:\nglemplate/renderer.RenderOptions\nglemplate/renderer RenderOptions","url":"glemplate/renderer.html#RenderOptions"},{"doc":"glemplate/renderer","title":"RenderResult","content":"pub type RenderResult =\n  Result(StringTree, RenderError)\n\n\nSynonyms:\nglemplate/renderer.RenderResult\nglemplate/renderer RenderResult","url":"glemplate/renderer.html#RenderResult"},{"doc":"glemplate/renderer","title":"StringifyError","content":"pub type StringifyError {\n  StringifyError\n}\n\nStringifyError\n\n\nSynonyms:\nglemplate/renderer.StringifyError\nglemplate/renderer StringifyError","url":"glemplate/renderer.html#StringifyError"},{"doc":"glemplate/renderer","title":"TemplateCache","content":"pub type TemplateCache =\n  Dict(ast.TemplateName, ast.Template)\n A cache of templates to use for rendering child templates.\n\nSynonyms:\nglemplate/renderer.TemplateCache\nglemplate/renderer TemplateCache","url":"glemplate/renderer.html#TemplateCache"},{"doc":"glemplate/renderer","title":"render","content":"pub fn render(\n  template: Template,\n  assigns: Dict(String, AssignData),\n  opts: RenderOptions,\n) -&gt; Result(StringTree, RenderError)\n Render given template with the assigns and options.\nSynonyms:\nglemplate/renderer.render\nglemplate/renderer render","url":"glemplate/renderer.html#render"},{"doc":"glemplate/renderer","title":"glemplate/renderer","content":" The renderer is given the template AST and some assigns, which it will\n render into a string output.","url":"glemplate/renderer.html"},{"doc":"glemplate/html","title":"encode","content":"pub fn encode(text: String) -&gt; StringTree\n Encode content as HTML entities so it&#39;s safe in an HTML template.\n\n Note! This is not safe for use in `&lt;script&gt;` or `&lt;style&gt;` tags!\nSynonyms:\nglemplate/html.encode\nglemplate/html encode","url":"glemplate/html.html#encode"},{"doc":"glemplate/html","title":"render","content":"pub fn render(\n  template: Template,\n  assigns: Dict(String, AssignData),\n  template_cache: Dict(String, Template),\n) -&gt; Result(StringTree, RenderError)\n Render an HTML template.\nSynonyms:\nglemplate/html.render\nglemplate/html render","url":"glemplate/html.html#render"},{"doc":"glemplate/html","title":"glemplate/html","content":" Utilities for rendering HTML templates.","url":"glemplate/html.html"},{"doc":"glemplate/parser","title":"ParseError","content":"pub type ParseError {\n  LexerError(String)\n  ParserError(List(String))\n}\n\nLexerError(String)\n\n\nParserError(List(String))\n\n\nSynonyms:\nglemplate/parser.ParseError\nglemplate/parser ParseError","url":"glemplate/parser.html#ParseError"},{"doc":"glemplate/parser","title":"Parser","content":"pub opaque type Parser\n\n\nSynonyms:\nglemplate/parser.Parser\nglemplate/parser Parser","url":"glemplate/parser.html#Parser"},{"doc":"glemplate/parser","title":"new","content":"pub fn new() -&gt; Parser\n Create a new parser.\n\n This same parser should be used for multiple operations to avoid creating\n the parser over and over again.\nSynonyms:\nglemplate/parser.new\nglemplate/parser new","url":"glemplate/parser.html#new"},{"doc":"glemplate/parser","title":"parse","content":"pub fn parse(\n  input: String,\n  parser: Parser,\n) -&gt; Result(List(Node), ParseError)\n Parse the tokens into a node list.\nSynonyms:\nglemplate/parser.parse\nglemplate/parser parse","url":"glemplate/parser.html#parse"},{"doc":"glemplate/parser","title":"parse_to_template","content":"pub fn parse_to_template(\n  input: String,\n  name: String,\n  parser: Parser,\n) -&gt; Result(Template, ParseError)\n Parse the input into a template with the given name.\nSynonyms:\nglemplate/parser.parse_to_template\nglemplate/parser parse_to_template","url":"glemplate/parser.html#parse_to_template"},{"doc":"glemplate/parser","title":"glemplate/parser","content":" The parser transforms a string input into a template AST.\n\n ## Template language description\n\n The template language uses `&lt;%` and `%&gt;` as the tag delimiters. For\n outputting, the start delimiter `&lt;%=` is used.\n\n Comments can be inserted with `&lt;%!-- Comment --%&gt;`, they are not emitted to\n the AST.\n\n Assigns can be referenced using their name (key in the assigns map). After\n an assign name, the contents of the assign can be accessed using field\n based access: `foo.wibble` accesses field `wibble` in assign `foo`.\n\n The accesses can be chained, e.g. `foo.wibble.wobble`. These access methods\n work in any place where an assign reference is expected.\n\n Any other content in the template is emitted as text nodes.\n\n ### Tags\n\n #### Output\n\n ```text\n &lt;%= foo %&gt;\n ```\n\n Emits an output node of the assign `foo`. The contents are encoded using\n the encoder given in the render call.\n\n #### Raw output\n\n ```text\n &lt;%= raw foo %&gt;\n ```\n\n Emits an output node of the assign `foo`. The contents are not encoded.\n\n #### If\n\n ```text\n &lt;% if foo %&gt;\n   ...\n &lt;% else %&gt;\n   ...\n &lt;% end %&gt;\n ```\n\n Emits an if node. If the assign `foo` is truthy (not `False`, 0, or empty),\n the first nodes are rendered, otherwise the nodes in the `else` block. The\n `else` block is optional.\n\n #### For\n\n ```text\n &lt;% for bar in foo %&gt;\n   ...\n &lt;% end %&gt;\n ```\n\n Emits a for node. Each item of `foo` is looped through, making it available\n as the assign `bar` inside the block. The contents of the block are\n rendered for each item.\n\n #### Render\n\n ```text\n &lt;% render tpl.txt.glemp %&gt;\n &lt;% render tpl.txt.glemp, name: user.name, level: user.level %&gt;\n ```\n\n Emits a render node. This will render another template in this place. The\n listed assigns will be the assigns given to the template. The name of the\n assign in the child template is on the left, the value is on the right.","url":"glemplate/parser.html"},{"doc":"glemplate/text","title":"encode","content":"pub fn encode(str: String) -&gt; StringTree\n An encoder that just returns the content as-is.\nSynonyms:\nglemplate/text.encode\nglemplate/text encode","url":"glemplate/text.html#encode"},{"doc":"glemplate/text","title":"render","content":"pub fn render(\n  template: Template,\n  assigns: Dict(String, AssignData),\n  template_cache: Dict(String, Template),\n) -&gt; Result(StringTree, RenderError)\n Render a plain text template.\nSynonyms:\nglemplate/text.render\nglemplate/text render","url":"glemplate/text.html#render"},{"doc":"glemplate/text","title":"glemplate/text","content":" Utilities for rendering plain text templates.","url":"glemplate/text.html"}]);